function echo ( data, title ) {
	// works only in debug mode with data present
	if ( !modes.debug || data == undefined ) return false;
	// browser or TV Box
	if ( stb ) {
		// simple data left as it is, complex - converted to json
		if ( data instanceof Object || data instanceof Array ) data = JSON.stringify(data, null, 4);
		// combine all together
		title = title || '';
		stb.Debug((title ? title + ': ' : '') + data);
	} else {
		// in browser debug mode so just print everything
		if ( window && window.console && window.console.info ) return window.console.info(data);
	}
	return true;
}

/**
 * `byID` is function for return element object
 *
 * @function
 * @name byID
 * @param {function(id):*}
 * @example
 * byID('frame');
 * @return [object]
 */
function byID(id) {return document.getElementById(id);}

/**
 * `random` is function for return random value between 0 and @value
 *
 * @function
 * @name random
 * @param {function(max):*}
 * @example
 * random(9999);
 * @return int
 */
function random(m,n){m = parseInt(m);n = parseInt(n);return Math.floor( Math.random() * (n - m + 1) ) + m;}




Array.prototype.shuffle = function( b )
{
    var i = this.length, j, t;
    while(i)
    {
        j = Math.floor( ( i-- ) * Math.random() );
        t = b && typeof this[i].shuffle!=='undefined' ? this[i].shuffle() : this[i];
        this[i] = this[j];
        this[j] = t;
    }
    return this;
};

/**
 * `log` is function for write log
 *
 * @function
 * @name log
 * @param {log(line):*}
 * @example
 * log('request from YouTube API : from 1 to 24, total items 97');
 * @return void
 */
function log(line) {
    if(modes.debug) {
        try{
            gSTB.Debug(typeof line == 'string' ? line : line.toString());
        } catch(e){
            try{
                console.log(typeof line == 'string' ? line : line.toString());
            } catch(e){

            }
        }
    }
}

/**
 * `string_replace` is function for replace parts of string
 *
 * @function
 * @name string_replace
 * @param {string_replace(search, replace, subject, count):*}
 * @example
 * string_replace('&amp;', '&', 'string &amp; me');
 * @return string
 */
function string_replace (search, replace, subject, count) {
    var i = 0, j = 0, temp = '', repl = '', sl = 0, fl = 0,
            f = [].concat(search),
            r = [].concat(replace),
            s = subject,
            ra = r instanceof Array, sa = s instanceof Array;
    s = [].concat(s);
    if (count) {
        this.window[count] = 0;
    }

    for (i=0, sl=s.length; i < sl; i++) {
        if (s[i] === '') {
            continue;
        }
        for (j=0, fl=f.length; j < fl; j++) {
            temp = s[i]+'';
            repl = ra ? (r[j] !== undefined ? r[j] : '') : r[0];
            s[i] = (temp).split(f[j]).join(repl);
            if (count && s[i] !== temp) {
                this.window[count] += (temp.length-s[i].length)/f[j].length;}
        }
    }
    return sa ? s : s[0];
}

/**
 * `print_r` is function for create string from object|array
 *
 * @function
 * @name print_r
 * @param {print_r(object|array):*}
 * @example
 * print_r(eval('{"main":{"one":1,"0":"zero"}}'));
 * @return string
 */
function print_r(arr, level) {
    var print_red_text = "";
    if(!level) level = 0;
    var level_padding = "";
    for(var j=0; j<level+1; j++) level_padding += "    ";
    if(typeof(arr) == 'object') {
        for(var item in arr) {
            var value = arr[item];
            if(typeof(value) == 'object') {
                print_red_text += level_padding + "'" + item + "' :\n";
                print_red_text += print_r(value,level+1);
        }
            else
                print_red_text += level_padding + "'" + item + "' => \"" + value + "\"\n";
        }
    }

    else  print_red_text = "===>"+arr+"<===("+typeof(arr)+")";
    return print_red_text;
}

/**
 * `is_numeric` is function checking on int value
 *
 * @function
 * @name is_numeric
 * @param {is_numeric(value):*}
 * @example
 * is_numeric('999');
 * @return bool
 */
function is_numeric(mixed_var) {
    return (typeof(mixed_var) === 'number' || typeof(mixed_var) === 'string') && mixed_var !== '' && !isNaN(mixed_var);
}

/**
 * `empty` is function checking on exits value
 *
 * @function
 * @name empty
 * @param {empty(value):*}
 * @example
 * empty('');
 * @return bool
 */
function empty (mixed_var) {
    if (mixed_var === "" ||
        //mixed_var === 0 ||
        //mixed_var === "0" ||
        mixed_var === null ||
        mixed_var === false ||
        typeof mixed_var === 'undefined' ||
        typeof mixed_var === 'NaN'
    ) {
        return true;
    }
    if (typeof mixed_var == 'object') {
        for (var key in mixed_var) {
            return false;
        }
        return true;
    }
    return false;
}

var trim = {
    "left":function(str){
        try {
            return str.replace(/^\s+/, '');
        } catch(e){return str;}
    },
    "right":function(str){
        try {
            return str.replace(/\s+$/, '');
        } catch(e){return str;}
    },
    "all":function(str){
        return this.right(this.left(str));
    },
    "spaces":function(str){
        try {
            return str.replace(/\s{2,}/g, ' ');
        } catch(e){return str;}
    }
};
/*
Array.prototype.min = function() {
    var min = this[0];
    var len = this.length;
    for (var i = 1; i < len; i++) {
        if (this[i] < min) {
            min = this[i];
        }
    }
    return min;
};
Array.prototype.minIndex = function() {
    var min = 0;
    var len = this.length;
    for (var i = 1; i < len; i++) {
        if (this[i] < min) {
            min = i;
        }
    }
    return min;
};
*/
function loadScript(src, callback) {
    //log(src);
    var apiRequestNode = document.createElement('script');
    apiRequestNode.src = src;
    apiRequestNode.type = 'text/javascript';
    document.documentElement.firstChild.appendChild(apiRequestNode);
    log('- - - - - -script "' + src + '" loaded');
    if(callback!='') {
        setTimeout(function(){eval(callback);}, 300);
    }
}
/**
 * `setEnvironmentValue` is function for set var, value to devise
 *
 * @function
 * @name setEnvironmentValue
 * @param {function():*}
 * @example
 * setEnvironmentValue();
 * @return void
 */
function setEnvironmentValue(name, value){
    log('Saving ' + name + ' = '+ value);
    stb.RDir('setenv ' + name + ' ' + value);
}
/**
 * `getEnvironmentValue` is function for get var value from devise
 *
 * @function
 * @name getEnvironmentValue
 * @param {function(name):*}
 * @example
 * getEnvironmentValue('quality');
 * @return string
 */
function getEnvironmentValue(name){
    return stb.RDir('getenv ' + name);
}
/**
 * `getXmlHttp` is function for create correctly XMLHttpRequest object for ajax
 *
 * @function
 * @name getXmlHttp
 * @param {function():*}
 * @example
 * getXmlHttp();
 * @return XMLHttpRequest
 */
function getXmlHttp() {
  if (typeof XMLHttpRequest === 'undefined') {
    XMLHttpRequest = function() {
      try {return new ActiveXObject("Msxml2.XMLHTTP.6.0");}
        catch(e) {}
      try {return new ActiveXObject("Msxml2.XMLHTTP.3.0");}
        catch(e) {}
      try {return new ActiveXObject("Msxml2.XMLHTTP");}
        catch(e) {}
      try {return new ActiveXObject("Microsoft.XMLHTTP");}
        catch(e) {}
      throw new Error("This browser does not support XMLHttpRequest.");
    };
  }
  return new XMLHttpRequest();
}
function dump(arr,level) {
	var dumped_text = "";
	if(!level) level = 0;
	
	//The padding given at the beginning of the line.
	var level_padding = "";
	for(var j=0;j<level+1;j++) level_padding += "    ";
	if (level > 10) return level_padding + "*Maximum Depth Reached*\n"; //Too much recursion preventer

	if(typeof(arr) == 'object') { //Array/Hashes/Objects 
		for(var item in arr) {
			var value = arr[item];
			
			if(typeof(value) == 'object') { //If it is an array,
				dumped_text += level_padding + "'" + item + "' ...\n";
				dumped_text += dump(value,level+1);
			} else {
				dumped_text += level_padding + "'" + item + "' => \"" + value + "\"\n";
			}
		}
	} else { //Stings/Chars/Numbers etc.
		dumped_text = "===>"+arr+"<===("+typeof(arr)+")";
	}
	return dumped_text;
}
function createHTMLTree(obj){
    var el = document.createElement(obj.tag);
    for(var key in obj.attrs) {
        if (obj.attrs.hasOwnProperty(key)){
            if(key!='html') {
                el.setAttribute(key, obj.attrs[key]);
            } else {
                el.innerHTML = obj.attrs[key];
            }
        }
    }
    if(typeof obj.child != 'undefined'){
        for(var i=0; i<obj.child.length; i++){
            el.appendChild(createHTMLTree(obj.child[i]));
        }
    }
    return el;
}